package.path = "/etc/nsg/share/lua/5.1/?.lua;"..package.path
package.cpath = "/etc/nsg/lib/lua/5.1/?.so;"..package.cpath

module(..., package.seeall)
local nsg = require'nsg'
local mimetypes = require "xavante.mime"

local nsgPageName = nsgPageName -- get from parent script
local urlDecode
local htmlEncode
local doCmdXML
local genOut
local bodyPrint
local fstr
local initPostData
local explodeToTable
local tokens
local selectLanguage
local initEnvData
local headPrint
local htmlDecode
local readpopen = nsg.readpopen
local version = nsg.getVersion()
local luaToHuman = nsg.luaToHumanUTF8

tokens = function (s, delim)
   local pos = 1
   return function()
             local f, l = string.find(s, '[^'..delim..']+', pos)
             if f then
                pos = l + 1
                return string.sub(s, f, l)
             else
                return nil
             end
          end
end
explodeToTable = function (path, delim)
   local pathTable = {}
   for s in tokens(path, delim) do
      table.insert(pathTable, s)
   end
   return pathTable
end

urlDecode = function(s)
               return (string.gsub (string.gsub (tostring(s or ""), "+", " "),
                                    "%%(%x%x)",
                                    function (str)
                                       return string.char (tonumber (str, 16))
                                    end )
                    )
            end

htmlEncode = function(s)
                return string.gsub( string.gsub (string.gsub (string.gsub (tostring(s or ""), "&", "&amp;"),
                                                              "\"",
                                                              "&quot;"),
                                                 "<",
                                                 "&lt;"),
                                    ">",
                                    "&gt;")
             end

htmlDecode = function(s)
                return string.gsub( string.gsub (string.gsub (string.gsub (tostring(s), "&qt;", ">"),
                                                              "&quot;",
                                                              "\""),
                                                 "&lt;",
                                                 "<"),
                                    "&amp;",
                                    "&")
             end

local hostName =  htmlEncode(readpopen("hostname"))

fstr = function (...)
          return table.concat({...})
       end


initPostData = function (http)
                  if nsgServer == "xavante" then
                     http.data = cgilua.POST
                     for k,v in pairs(http.data) do
                        if type(v)=="table" and v.file then
                           local buf = "XX"
                           v.tmpFile = os.tmpname()
                           local f = io.open(v.tmpFile,"w")
                           while type(buf)=='string' and #buf>0 do
                              buf = v.file:read(1024*1024)
                              f:write(buf or "")
                           end
                           f:close()
                        end
                     end
                  else
                     if( http.env.request_method == 'POST' and
                         -- string.match(http.env.content_type,'application\/x-www-form-urlencoded') and
                         tonumber(http.env.content_length) > 0 )
                     then
                        local x=0
                        -- !!!  Don`t read more then http.env.content_length
                        for line in io.lines() do
                           http.raw_post = http.raw_post .."\010".. line;
                           for s1,s2 in string.gmatch('&'..line..'&', '([^&=]*)%=([^&]*)') do
                              http.data[urlDecode(s1)] = urlDecode(s2)
                           end
                           x = x + string.len(line)+1
                           if x >  tonumber(http.env.content_length) then
                              break
                           end
                        end
                     end
                   end
                     for s1,s2 in string.gmatch((http.env.query_string or "") ..'&', '([^&=]*)%=([^&=]*)') do
                        http.data[s1] = s2
                     end
               end

selectLanguage = function (http_lang)
   local langTokens = explodeToTable(http_lang or "ru", ",")
   local lang = {};
   local maxLang = "ru"
   local maxQ = 0
   local l,q
   for k,v in ipairs(langTokens) do

      l, q = string.match( string.gsub(v, "%s", "") , "([a-zA-Z][a-zA-Z])([;]*)([q=]*)([0-9.]*)")

      if((q == nil) or (q=="")) then
         q =  1000
      else
         q = string.match(q, ";q=([0-9.]*)" )
         if((q == nil) or (q=="")) then
            q =  1000
         else
            q = tonumber(q)
         end
      end
      l = l or "en"
      --print("//".. l .."~".. tostring(q))
      if(type(lang[l]) == "number") then
         lang[l] = ((tonumber(q) > lang[l]) and q) or lang[l]
      else
         lang[l] = tonumber(q)
      end

      if maxQ < lang[l] then
         maxQ = lang[l]
         maxLang = l
      end
   end
   return maxLang
end

initEnvData = function (http)
                 if (nsgServer == "xavante") then
                    http.env['lang'] = selectLanguage(cgilua.servervariable("HTTP_ACCEPT_LANGUAGE"))
                    http.env['auth_type'] =     cgilua.servervariable("AUTH_TYPE")
                    http.env['gateway_interface'] = cgilua.servervariable("GATEWAY_INTERFACE")
                    http.env['path'] =          cgilua.servervariable("PATH")
                    http.env['path_info'] =     cgilua.servervariable("PATH_INFO")
                    http.env['pwd'] =           cgilua.servervariable("PWD")
                    http.env['query_string'] =  cgilua.servervariable("QUERY_STRING")
                    http.env['remote_addr'] =   cgilua.servervariable("REMOTE_ADDR")
                    http.env['remote_port'] =   cgilua.servervariable("REMOTE_PORT")
                    http.env['remote_user'] =   cgilua.servervariable("REMOTE_USER")
                    http.env['request_method'] = cgilua.servervariable("REQUEST_METHOD")
                    http.env ['request_uri'] =   cgilua.servervariable("REQUEST_URI")
                    http.env['script_filename'] =cgilua.servervariable("SCRIPT_FILENAME")
                    http.env['script_name'] =   cgilua.servervariable("SCRIPT_NAME")
                    http.env['server_port'] =   cgilua.servervariable("SERVER_PORT")
                    http.env['server_name'] =   cgilua.servervariable("SERVER_NAME")
                    http.env['server_protocol'] = cgilua.servervariable("SERVER_PROTOCOL")
                    http.env['server_software'] = cgilua.servervariable("SERVER_SOFTWARE")
                    http.env['content_type'] =  cgilua.servervariable("CONTENT_TYPE")
                    http.env['content_length'] =cgilua.servervariable("CONTENT_LENGTH")
                   -- http.env['cookie'] =        parseCookie(cgilua.servervariable("HTTP_COOKIE") or "")
                 else
                    http.env['lang'] = selectLanguage(os.getenv("HTTP_ACCEPT_LANGUAGE"))
                    http.env['auth_type'] =     os.getenv("AUTH_TYPE")
                    http.env['gateway_interface'] = os.getenv("GATEWAY_INTERFACE")
                    http.env['path'] =          os.getenv("PATH")
                    http.env['path_info'] =     os.getenv("PATH_INFO")
                    http.env['pwd'] =           os.getenv("PWD")
                    http.env['query_string'] =  os.getenv("QUERY_STRING")
                    http.env['remote_addr'] =   os.getenv("REMOTE_ADDR")
                    http.env['remote_port'] =   os.getenv("REMOTE_PORT")
                    http.env['remote_user'] =   os.getenv("REMOTE_USER")
                    http.env['request_method'] = os.getenv("REQUEST_METHOD")
                    http.env ['request_uri'] =   os.getenv("REQUEST_URI")
                    http.env['script_filename'] = os.getenv("SCRIPT_FILENAME")
                    http.env['script_name'] =   os.getenv("SCRIPT_NAME")
                    http.env['server_port'] =   os.getenv("SERVER_PORT")
                    http.env['server_name'] =   os.getenv("SERVER_NAME")
                    http.env['server_protocol'] = os.getenv("SERVER_PROTOCOL")
                    http.env['server_software'] = os.getenv("SERVER_SOFTWARE")
                    http.env['content_type'] =  os.getenv("CONTENT_TYPE")
                    http.env['content_length'] = os.getenv("CONTENT_LENGTH")
                    --http.env['cookie'] =        parseCookie(os.getenv("HTTP_COOKIE") or "")
                 end
              end

headPrint = function(http, param, value)
               if param ~= nil then
                  http.head[param] = value
               end
            end

bodyPrint = function(http, text, ...)
               if text ~= nil then
                  http.body.text = http.body.text..tostring(text)
                  http:bodyPrint(...)
               end
            end


genOut = function(http)
            http:headPrint("NSGVersion", version)
            if #http.body.text==0 then http.body.text = "\0" end
            if nsgServer == "xavante" then
	      -- cgilua.header("X1",nsg.serialize(http.head["Content-Type"]))
			     
	       _, _, n1, n2 = (http.head["Content-Type"] or "text/html"):find("^([^/]*)/([^/]*)$")       
               cgilua.contentheader(n1, n2)
	       http.head["Content-Type"] = nil
	       
               cgilua.header("Cache-Control","max-age=798001");
              -- cgilua.header("Cache-Control", "post-check=0, pre-check=0");
               for k,v in pairs(http.head) do
                  cgilua.header(k, v)
               end
               cgilua.put(http.body.text)
            else
               io.write("HTTP/1.0  "..tostring(http.head["Status"] or 200).." OK".."\013\010")
               http:headPrint("Cache-Control", "max-age=798000")

               for k,v in pairs(http.head) do
                  if k ~= "Status" then
                     io.write(k,":",v,"\013\010");
                  end
               end
               io.write("\013\010");
               io.write(http.body.text)
            end
         end

function mimefrompath (path)
        local _,_,exten = string.find (path, "%.([^.]*)$")
        if exten then
                return mimetypes[exten]
        else
                return nil
        end
end

local cdexists, cdlist = pcall ( require, 'helpcd')

function unpackHelp(http)
   http.data.helppath = http.data and http.data.helppath and #http.data.helppath>0 and http.data.helppath or "_index.html"
   if cdexists then
      local mime = mimefrompath(http.data.helppath) or "text/html"
      http:headPrint("Content-Type", mime)
      local helppath=urlDecode(http and http.data and http.data.helppath or "_index.html")
      local idx=cdlist[helppath]or cdlist["_index.html"]
      http:bodyPrint(nsg.readpopen('tar -xf /usr/share/www/helpcd.tar -O '..idx[1]..'.txz | tar -xJ -O '..idx[2]))
   else
      http:bodyPrint('<meta http-equiv="refresh" content="0; url=http://www.nsg.ru/help/'..(http.data.helppath):gsub("[.]html$",".php")..'">')
      --http:headPrint("Status", 301)
      --http:headPrint("Location", "http://www.nsg.ru/help/"..http.data.helppath)
   end
end

function main()
   local http = {
      ['env']  = {},

      ['head'] = {
      },

      ['body'] = {
         ['text'] = ""
      },

      ['data'] = {},

      ['raw_post'] = '',
      headPrint = headPrint,
      bodyPrint = bodyPrint
   }

   initEnvData(http)
   initPostData(http)

   if http.env['request_method']:match("[gG][eE][tT]") then
      unpackHelp(http)
      genOut(http)
   end
end
